
#ifndef __CERBERUS_H
#define __CERBERUS_H

#include "landscape.h"

// lua headers
extern "C" {
	#include <lua.h>
	#include <lualib.h>
	#include <lauxlib.h>
}

// irrlicht headers
#ifdef USE_IRRKLANG
	#include "irrKlang.h"
#endif
#include <irrlicht.h>
using namespace irr;

// forward declarations
class mt19937;

class cConfiguration
{
public:
  bool loadFromFile( lua_State* L, const char* filename );   //!< load settings from lua file

public:
  bool fullscreen;     //!< true for fullscreen mode
  s32 width;           //!< width of screen/window
  s32 height;          //!< height of screen/window
  bool vsync;          //!< true for vertical synchronisation
  u32 screenDepth;     //!< bits per pixel if fullscreen mode
  size_t driverType;   //!< driver type (OpenGL, ...)

  double max_fps;
  double fps_period;
};

  
class cAudioDevice
{
public:
  cAudioDevice( std::string mediaPath );
  ~cAudioDevice( void );

  // set position of listener
  void setListenerPosition( const irrklang::vec3df& pos, const irrklang::vec3df& lookdir,
                            const irrklang::vec3df& velPerSecond=irrklang::vec3df(0, 0, 0),
                            const irrklang::vec3df& upVector=irrklang::vec3df(0, 1, 0) );

private:
#ifdef USE_IRRKLANG
	irrklang::ISoundEngine* irrKlang;	
	irrklang::ISoundSource* testSound;
#endif
};

// printf-like function
char* formatString( char* format, ... );

#endif // __CERBERUS_H
