#include "IrrConsole/testApp.h"
//=========================================================================
#ifdef WIN32
	#ifdef _DEBUG
		#pragma comment(lib,"IrrConsole-DEBUG")
	#else
		#pragma comment(lib,"IrrConsole")
	#endif
#endif
//=========================================================================
void IC_RunTestApp(int argc,char **argv);
void IC_TestParser();
//=========================================================================
int main(int argc,char **argv)
{
	IC_RunTestApp(argc,argv);
	return 0;
}
//=========================================================================
void IC_RunTestApp(int argc,char **argv)
{
	IC_TestApp app (argv[0]);
	array<String> args;
	for(s32 i = 1; i < argc; i++)
	{
		args.push_back(argv[i]);
	}
	try
	{
		app.main(args);
	}
	catch(IC_Error& err)
	{
		cerr<<endl;
		cerr<<"Error Caught ::"<<endl;
		cerr<<"  (Type = "<<err.getType()<<")"<<endl;
		cerr<<"  (Message = "<<err.getMessage()<<")"<<endl;
		cerr<<"  (Code = "<<err.getCode()<<")"<<endl;
		cerr<<endl;
	}
}
//=========================================================================
void IC_TestParser()
{
	//TODO: for testing only
	WideString cmdLine = L"echo \"Hello World\" du\\de h\\\"o\\\"w";
	array<WideString> args;
	WideString cmdName = L"";
	IC_CmdLineParser parser(cmdLine);
	parser.parse(cmdName,args);
	cout<<"Command Line : ["<<cmdLine<<"]"<<endl;
	cout<<"Command Name : ["<<cmdName<<"]"<<endl;
	cout<<"Arguments : ["<<args.size()<<"]"<<endl;
	for(u32 x = 0; x < args.size(); x++)
	{
		cout<<" arg["<<x<<"]==>{"<<args[x]<<"}"<<endl;
	}
}
//=========================================================================
IC_TestApp::IC_TestApp(const String str) : device(0), 
										   name(str), 
										   sceneNode(0), 
										   levelMesh(0), 
										   modelMesh(0),
										   modelTexture(0),
										   testTexture(0),
										   captionFont(0)
{
}
//=========================================================================
IC_TestApp::~IC_TestApp()
{
	if(device)
	{
		device->drop();
		device = 0;
	}
}
//=========================================================================
bool IC_TestApp::OnEvent(irr::SEvent event)
{
	if(event.EventType == irr::EET_KEY_INPUT_EVENT )
	{
		if(event.KeyInput.PressedDown)
		{
			if(event.KeyInput.Key == irr::KEY_ESCAPE)
			{
				if(console.isVisible())
				{
					console.setVisible(false);
					return true;
				}
				else
				{
					setRunning(false);
					return true;
				}
			}
			else if(event.KeyInput.Key == IC_Console::IC_KEY_TILDE)
			{
				if(!console.isVisible())
				{
					console.setVisible(true);
					return true;
				}
				else if(!event.KeyInput.Control)
				{
					console.setVisible(false);
					return true;
				}
				
			}

			if(console.isVisible())
			{
				console.handleKeyPress(event.KeyInput.Char, event.KeyInput.Key,event.KeyInput.Shift, event.KeyInput.Control);
				return true;
			}

		}


	}
	else if(event.EventType == irr::EET_LOG_TEXT_EVENT)
	{
		console.logMessage_ANSI(event.LogEvent.Level,event.LogEvent.Text);
		return true;
	}
	else if(event.EventType == irr::EET_MOUSE_INPUT_EVENT)
	{
		return console.isVisible();
	}
	return false;
}
//=========================================================================
void IC_TestApp::main(array<String>& args)
{
	//initialize the app
	init(args);
	
	//set running to true
	setRunning(true);
	while(device && device->run() && isRunning())
	{
		if(!run())
		{
			setRunning(false);
		}
	}

}
//=========================================================================

void IC_TestApp::init(array<String>& args)
{
	irr::video::E_DRIVER_TYPE driverType = irr::video::EDT_DIRECTX9;
	s32 bitsPerPixel = 32;
	dimension2d<s32> screenDim(800,600);
	bool fullscreen = false;
	bool stencil = true;
	bool vsync = true;
	device = irr::createDevice(driverType, screenDim, bitsPerPixel, fullscreen,stencil,vsync, this);
	if(!device)
	{
		throw IC_Error(L"could not create irrlicht device");
	}
	
	//load the caption font
	captionFont = device->getGUIEnvironment()->getFont("data/font/caption.bmp");
	if(!captionFont)
	{
		captionFont = device->getGUIEnvironment()->getBuiltInFont();
	}
	//initialize the caption array
	captionText.push_back(L"IrrConsole : Quake Style Drop Down Console Demo For Irrlicht");
	captionText.push_back(L"Author : Saurav Mohapatra (mohaps@gmail.com)");
	captionText.push_back(L"");
	captionText.push_back(L"HELP TEXT:");
	captionText.push_back(L"============================================================");
	captionText.push_back(L"     Press the tilde (~) key to toggle console");
	captionText.push_back(L"     Use the UP/DN arrow keys to access command history");
	captionText.push_back(L"     Type commands at the prompt");
	captionText.push_back(L"     To execute a command put \\ in front");
	captionText.push_back(L"     e.g. \\list");
	captionText.push_back(L"     use the \"list\" or \"help\" commands to find out more");
	captionText.push_back(L"     to start try the command \"\\help show_node\");");
	captionText.push_back(L"============================================================");
	captionText.push_back(L" ");
	captionText.push_back(L"TO QUIT THIS DEMO PRESS ESCAPE KEY");
	
	
	//initialize the caption line height
	captionHeight = captionFont->getDimension(L"X").Height + 2;
	//initialize the console
	//this is how you alter some of the config params
	console.getConfig().dimensionRatios.Y = 0.8f;
	//now initialize
	console.initializeConsole(device->getGUIEnvironment(),screenDim);

	//register common commands
	console.loadDefaultCommands(device);

	//register custom commands
	// we register the show node command
	console.registerCommand(new TestCommand_SHOWNODE(this));
	// we register the hide node command
	console.registerCommand(new TestCommand_HIDENODE(this));

	//we do some app specific init her
	//to save time we preload the mesh, map and texture required for this demo

	//for the texture
	testTexture = device->getVideoDriver()->getTexture("data/texture/test.jpg");

	//for the model
	modelMesh = device->getSceneManager()->getMesh("data/model/faerie.md2");
	modelTexture = device->getVideoDriver()->getTexture("data/model/faerie.bmp");

	//for the map
	device->getFileSystem()->addZipFileArchive("data/pak/map-20kdm2.pk3");
	levelMesh = device->getSceneManager()->getMesh("20kdm2.bsp");

	//init the scene node to 0
	sceneNode = 0;

	device->getSceneManager()->addCameraSceneNodeFPS();

	//for starters we have displayed the quake III level
	showModel();

	

	
	
}
bool IC_TestApp::isRunning()
{
	return bRunning;
}
void IC_TestApp::setRunning(bool bVal)
{
	bRunning = bVal;
}
bool IC_TestApp::run()
{

	irr::video::IVideoDriver* videoDriver = device->getVideoDriver();
	irr::scene::ISceneManager* sceneManager = device->getSceneManager();
	irr::gui::IGUIEnvironment* guienv = device->getGUIEnvironment();
	s32 fps = videoDriver->getFPS();
	u32 deltaMillis = 0;
	if(fps > 0)
	{
		deltaMillis  = (u32) (1000.0f / (f32)fps);
	}
	
	videoDriver->beginScene(true,true,irr::video::SColor(200,10,10,100));
		sceneManager->drawAll();
		guienv->drawAll();
		console.renderConsole(guienv,videoDriver,deltaMillis);
		//if console is not visible
		//we draw the caption text
		if(!console.isVisible())
		{
			drawCaptions();
		}
	videoDriver->endScene();
	
	return true;
}
void IC_TestApp::showModel()
{
	hideNode();
	if(modelMesh)
	{
		//create an animated mesh scene node
		sceneNode = device->getSceneManager()->addAnimatedMeshSceneNode(modelMesh);
		//set the texture
		sceneNode->setMaterialTexture(0, modelTexture);
		//set lighting to false
		sceneNode->setMaterialFlag(irr::video::EMF_LIGHTING,false);
		//set the model position
		sceneNode->setPosition(vector3df(0,10,80));
		//set the frame loop
		((irr::scene::IAnimatedMeshSceneNode *)sceneNode)->setFrameLoop(0, modelMesh->getFrameCount());
	}
	else
	{
		throw IC_Error(L"Model could not be loaded");
	}
	device->setWindowCaption(L"IrrConsole Demo App [ Showing MD2 Model]");
}
void IC_TestApp::showMap()
{
	hideNode();
	if(levelMesh)
	{
		//add the quake III level
		sceneNode = device->getSceneManager()->addOctTreeSceneNode(levelMesh->getMesh(0));
		//set lighting to false
		sceneNode->setMaterialFlag(irr::video::EMF_LIGHTING,false);
		//position camera in a suitable place
		sceneNode->setPosition(vector3df(-1300,-144,-1249));
	}
	else
	{
		throw IC_Error(L"Level/Map could not be loaded");
	}
	device->setWindowCaption(L"IrrConsole Demo App [ Showing Quake III Map]");
}
void IC_TestApp::showTestNode()
{
	hideNode();
	if(testTexture)
	{
		//add a test scene node
		sceneNode = device->getSceneManager()->addTestSceneNode(20);
		//set the texture
		sceneNode->setMaterialTexture(0, testTexture);
		//set lighting to false
		sceneNode->setMaterialFlag(irr::video::EMF_LIGHTING,false);

		//set the model position
		sceneNode->setPosition(vector3df(0,0,30));

		//create a rotation animator
		irr::scene::ISceneNodeAnimator* anim = device->getSceneManager()->createRotationAnimator(vector3df(1,2,1));
		sceneNode->addAnimator(anim);
		anim->drop();

	}
	else
	{
		throw IC_Error(L"Test Texture Could not be loaded");
	}
	device->setWindowCaption(L"IrrConsole Demo App [ Showing Test Node]");
}
void IC_TestApp::hideNode()
{
	if(sceneNode)
	{
		sceneNode->remove();
		sceneNode = 0;
	}
	device->setWindowCaption(L"IrrConsole Demo App [ Node Hidden ]");
}
void IC_TestApp::drawCaptions()
{
	static const irr::video::SColor fontColor(200,200,200,200);
	dimension2d<s32> screenDim = device->getVideoDriver()->getScreenSize();
	rect<s32> lineRect(10,10,screenDim.Width, screenDim.Height);
	for(u32 i = 0; i < captionText.size(); i++)
	{
		captionFont->draw(captionText[i].c_str(),lineRect,fontColor);
		lineRect.UpperLeftCorner.Y += captionHeight;
		lineRect.LowerRightCorner.Y += captionHeight;
	}
}
