

#ifndef __LANDSCAPE_H
#define __LANDSCAPE_H

// standard headers
#include <iostream>
#include <cmath>

// Mersenne Twister RNG
#include "mt19937.hpp"

// irrlicht headers
#include "irrlicht.h"
using namespace irr;

// cerberus headers
#include "entities.h"


class cWorld {
public:
  cWorld( IrrlichtDevice* device, int level, int size, int energy, double scalarStart, double hScalar );
  ~cWorld();
	
  void print();
  void create();
  void showStartScreen();
  scene::ISceneNode* getNode()  { return mNode; }
  cSoul* getSoul()  { return mSoul; }
	void CheckSelection();
  
  scene::IAnimatedMesh* getTreeMesh()
    { return mTreeMesh; }
  scene::IAnimatedMesh* getBoulderMesh()
    { return mBoulderMesh; }
  scene::IAnimatedMesh* getRobotMesh()
    { return mRobotMesh; }
	core::stringw GetName()
		{ return mName; }

	scene::ISceneManager* getSceneManager()
		{ return mSmgr; }

	cEntityBase* cWorld::findEntity();
	bool createEntity( eEntity entityType, cEntityBase* entityBelow );
	int absorbEntity( cEntityBase* entityBelow );  /**< Absorb entity above entityBelow */
	void hyperjump();
	bool doTransferSoul( cEntityBase* robot );
	bool transferSoul( cEntityBase* entityBelow );
    
  inline double getRandomNumber()
    { return rng->genrand_real1(); }
		
  // static function to check code
  static bool checkLandscapeCode( unsigned int level, unsigned int code );
    
private:
  void init();  
  double gaussian( double x0, double sigma );
  double gaussian_4( double sigma, double a1, double a2, double a3, double a4 );
  double gaussian_3( double sigma, double a1, double a2, double a3 );
  
private:
  // preloaded meshes and icons 
  scene::IAnimatedMesh* mTreeMesh;       //!< Tree mesh
  scene::IAnimatedMesh* mBoulderMesh;    //!< Boulder mesh
  scene::IAnimatedMesh* mRobotMesh;      //!< Robot mesh

	IrrlichtDevice* mDevice;
  video::IVideoDriver* mDriver;
	scene::ISceneManager* mSmgr;
	gui::IGUIEnvironment* mEnv;
  ILogger *mLogger;

	unsigned int mLevel;   //!< which level is this landscape
  int mSize;            //!< size of land
  int mEnergy;          //!< available energy for this world
  double mScalarStart;
  double mHScalar;
  scene::ISceneNode* mNode;
  scene::ICameraSceneNode* mCamera;
  cSoul *mSoul;                    //!< robot with soul
	core::stringw mName;   //!< "name" of landscape, i.e. "landscape 0653"

  double** mPoints;     //!< land information
	cElement*** mElements; //!< floor elements
  mt19937* rng;          //!< pointer to RNG
};

#endif
